// The global instance of our data storage class
Utils::Data::Storage@ g_DataStorage = null;

// The global instance of our logic class
UltrawideUiFix::Fixer@ g_Fixer = null;

// The global frame counter
uint g_FrameCounter = 0;

// Main entry point
void Main() {
	// Instantiate our data storage class
	@g_DataStorage = Utils::Data::Storage('db.sqlite');

	// Instantiate our transformations
	array<UltrawideUiFix::Transformations::BaseTransformation@> transformations = {
		// Chat, bottom left
		UltrawideUiFix::Transformations::OverlayFrameMoveTransformation('FrameInGameBase', array<string> = {'FrameChat'}, UltrawideUiFix::MOVE_LEFT),
		UltrawideUiFix::Transformations::OverlayFrameMoveTransformation('FrameInGameBase', array<string> = {'FrameChat', 'FrameChatUnfolded'}, UltrawideUiFix::MOVE_RIGHT),
		// Network problem icon, bottom right
		UltrawideUiFix::Transformations::OverlayFrameMoveTransformation('FrameInGameBase', array<string> = {'#0', 'QuadNetworkProblem'}, UltrawideUiFix::MOVE_RIGHT),
		// Hotseat mode player times, top right
		UltrawideUiFix::Transformations::MoveTransformation('UIModules_HotSeat_InGame', '#frame-hotseatplayers', UltrawideUiFix::MOVE_LEFT),
		// Countdown, right
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_Countdown', '#Race_Countdown', UltrawideUiFix::MOVE_RIGHT),
		// Messages, top left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_DisplayMessage', '#Race_DisplayMessage', UltrawideUiFix::MOVE_LEFT),
		// Lap counter, top right
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_LapsCounter', '#Race_LapsCounter', UltrawideUiFix::MOVE_RIGHT),
		// Slide out medal banner, top left
		UltrawideUiFix::Transformations::AnimationMoveTransformation('UIModule_Race_Record', 'AnimMgr\\.Add\\(Controls\\.Frame_Medal, "<a pos=\\\\"([0-9.-]+) ([0-9.-]+)\\\\"', 'AnimMgr.Add(Controls.Frame_Medal, "<a pos=\\"%X %Y\\"', UltrawideUiFix::MOVE_LEFT),
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_Record', '#frame-medal', UltrawideUiFix::MOVE_LEFT),
		UltrawideUiFix::Transformations::ExpandTransformation('UIModule_Race_Record', '#clip-medal-banner'),
		// Records, left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_Record', '#Race_Record', UltrawideUiFix::MOVE_LEFT),
		// Respawn & Give up prompts, right
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_RespawnHelper', '#Race_RespawnHelper', UltrawideUiFix::MOVE_RIGHT),
		// Spectator mode picker, bottom right
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_SpectatorBase', '#Race_SpectatorBase_Commands', UltrawideUiFix::MOVE_RIGHT),
		// Checkpoint times, varies between offline and online
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_TimeGap', '#Race_TimeGap', UltrawideUiFix::MOVE_AUTO),
		// Warmup information, right
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Race_WarmUp', '#Race_WarmUp', UltrawideUiFix::MOVE_RIGHT),
		// Small score table, left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Rounds_SmallScoresTable', '#Rounds_SmallScoresTable', UltrawideUiFix::MOVE_LEFT),
		// Best & previous times, right
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_TimeAttack_BestRaceViewer', '#Race_BestRaceViewer', UltrawideUiFix::MOVE_RIGHT),

		// COTD
		// Qualification information, left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_COTDQualifications_QualificationsProgress', '#COTDQualifications_QualificationsProgress', UltrawideUiFix::MOVE_LEFT),
		UltrawideUiFix::Transformations::ExpandTransformation('UIModule_COTDQualifications_QualificationsProgress',  '#frame-global'),
		// Qualification results, left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_COTDQualifications_Ranking', '#COTDQualifications_Ranking', UltrawideUiFix::MOVE_LEFT),
		UltrawideUiFix::Transformations::ExpandTransformation('UIModule_COTDQualifications_Ranking', '#frame-global'),
		// Knockout results, left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Knockout_KnockoutInfo', '#Knockout_KnockoutInfo', UltrawideUiFix::MOVE_LEFT),

		// Royal
		// top right, recently finished players
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Royal_FinishFeed', '#Royal_FinishFeed', UltrawideUiFix::MOVE_RIGHT),
		// top left, team ranking
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Royal_LiveRanking', '#Royal_LiveRanking', UltrawideUiFix::MOVE_LEFT),
		// right, Restart prompt
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Royal_RespawnHelper', '#Royal_RespawnHelper', UltrawideUiFix::MOVE_RIGHT),
		// top right, team results
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Royal_TeamScore', '#Royal_TeamScore', UltrawideUiFix::MOVE_RIGHT),

		// Ranked
		// Player positions and points, left
		UltrawideUiFix::Transformations::MoveTransformation('UIModule_Teams_Matchmaking_LiveRanking', '#Teams_Matchmaking_LiveRanking', UltrawideUiFix::MOVE_LEFT)
	};

	// Instantiate our logic class
	@g_Fixer = UltrawideUiFix::Fixer(transformations);
}

// Render function called every frame
void Render() {
	// Call the Render function of our fixer instance every 10th frame to make sure we don't check for UI changes too often
	if (++g_FrameCounter >= 9) {
		g_Fixer.Render();
		g_FrameCounter = 0;
	}
}
